/* Written by Elizabeth S. Odekirk
 *
 * Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */


/*
<applet 
    CODEBASE=".."
    CODE="hamlet.AppButton.class" NAME="BlockStacker.BlockStacker"
    width=160 height=25 align=absbottom>
    <param name=LABEL value="Block Stacker Simulation">
    A Java applet should appear here
</applet>
*/

package BlockStacker;

import java.awt.*;
import hamlet.*;
import BlockStacker.*;


public class BlockStacker extends AppFrame {

    MenuItem addBottomButton;
    MenuItem addButton;
    MenuItem remButton;
    WorkArea centerPanel;
    
    public BlockStacker () {

	setTitle("Block Stacker");
	setLayout(new BorderLayout());
	MenuBar menuBar = getMenuBar();

	Menu blocksMenu = new Menu("Blocks");
	addButton = new MenuItem("Add Block to Top");
	addBottomButton = new MenuItem("Add Block to Bottom");
	remButton = new MenuItem("Remove Block");
	blocksMenu.add(addButton);
	blocksMenu.add(addBottomButton);
	blocksMenu.add(remButton);
	menuBar.add(blocksMenu);

	centerPanel = new WorkArea();
	add("Center", centerPanel);

        helpText =
	    " This applet simulates the block-stacking problem\n" +
	    " described in Chapter 4 of \"Introduction to Scientific\n" +
	    " Programming\" by Joseph L. Zachary.\n\n" +

	    " The blocks and the table are always of the same width\n" +
	    " You can use the mouse to adjust that size as well as to\n" +
	    " move the blocks back and forth.\n\n" +

	    " All measurements of block sizes and positions are made\n" +
	    " in pixels.  A pixel is the smallest unit into which\n" +
	    " your computer's display can be divided.\n\n" +

            " File Menu\n" +
	    "   + Reset:  Removes all blocks from table\n" +
            "   + Refresh: Refreshes the applet\n" +
	    "   + Hide:  Removes the applet from the display\n" +
	    "   + Quit:  Terminates the applet\n\n" +

	    " Blocks Menu\n" +
	    "   + Add Block to Top:  Adds block to top of stack\n" +
	    "   + Add Block to Bottom:  Adds block to bottom of stack\n" +
	    "   + Remove Block:  Removes block from top of stack\n\n"+

	    " Help Menu\n" +
	    "   + With Applet:  Displays this help dialog\n\n" +

            " Mouse Actions\n" +
	    "   + Drag table to resize table/block\n" +
	    "   + Drag block to move it and all blocks it supports\n\n" +

	    " Displayed Information\n" +
	    "   + Table/block size in pixels is displayed on table\n" +
	    "   + Current overhang in pixels of block stack beyond\n" +
	    "       right edge of table is displayed on table\n" +
	    "   + Offset in pixels of right edge of each block beyond\n" +
	    "       right edge of its support is displayed on block\n" +
            "   + A block is displayed in green if the combined centers\n" +
            "       of gravity of the block and all the blocks that it\n" +
            "       it supports is above the surface below.  Otherwise,\n" +
            "       the block is displayed in red\n\n" +

            " (24Sep96)";

	pack();

    }


    public void full (boolean b) {
	addButton.enable(!b);
	addBottomButton.enable(!b);
    }

    public void empty (boolean b) {
	remButton.enable(!b);
    }
    

    public void reset () {
	centerPanel.reset();
    }


    protected void handleMessage (String msg) {
    	if (msg.equals("add"))
	    centerPanel.addBlock();
	else if (msg.equals("remove"))
	    centerPanel.remBlock();
	else if (msg.equals("oneHalfway"))
	    centerPanel.oneHalfway();
	else if (msg.equals("twoThirds"))
	    centerPanel.twoThirds();
	else if (msg.equals("custom"))
	    centerPanel.custom();
    }

    public boolean action (Event theEvent, Object arg) {

	if (super.action(theEvent, arg)) return(true);

	if (theEvent.target instanceof MenuItem) {

	    if (theEvent.target.equals (addButton)) {
		centerPanel.addBlock();
		return true;
	    }

	    if (theEvent.target.equals (addBottomButton)) {
		centerPanel.addBlockBottom();
		return true;
	    }

	    if (theEvent.target.equals (remButton)) {
		centerPanel.remBlock();
		return true;
	    }

	}

	return(false);

    }
}
